/*
 * Datenbanksysteme (VL 4.0, 181.186)
 * Laboruebungsteil, WS 2009/2010
 *
 * Beispiel 3
 */

import java.sql.*;
import java.math.BigDecimal;

public class Bestellungsverwaltung
{
  private Connection dbconn = null;

  public void dbConnect()
  {
    String user = System.getProperty("user.name");
    String pass = "bCZ20yJg";
    String url  = "jdbc:postgresql:" + user;

    try
    {
      if (dbconn != null)
        dbDisconnect();

      Class.forName("org.postgresql.Driver");
      dbconn = DriverManager.getConnection(url, user, pass);
      dbconn.setAutoCommit(false);
    }
    catch(Exception ex)
    {
      System.out.println(ex.toString());
    }
  }

  public void dbOutputKunden()
  {
    Statement stmt = null;
    ResultSet res = null;

    try
    {
      if (dbconn == null)
        dbConnect();

      stmt = dbconn.createStatement();
      res = stmt.executeQuery(
        "SELECT " +
        "  k.knr, k.vname, k.nname, k.tpunkte, COUNT(b.bid) AS ordercnt, " +
        "  SUM(f_bestellwert(b.bid)) AS sumorders, AVG(f_bestellwert(b.bid)) AS avgorders "+
        "FROM Kunde AS k " +
        "  LEFT JOIN ( " +
        "    Bestellung AS b " +
        "    JOIN bezahlt AS bez ON (bez.bid = b.bid) " +
        "  ) ON (k.knr = b.kunde) " +
        "GROUP BY k.knr, k.vname, k.nname, k.tpunkte " +
        "ORDER BY k.knr ASC");

      ResultSetMetaData rsmd = res.getMetaData();
      int cols = rsmd.getColumnCount();
      while(res.next())
      {
        int i;
        for(i = 1; i < cols; ++i)
          System.out.print(res.getString(i) + ",");
        System.out.println(res.getString(i));
      }

      res.close();
      stmt.close();
    }
    catch(SQLException ex)
    {
      System.out.println(ex.toString());

      try
      {
        if (res != null)
          res.close();
        if (stmt != null)
          stmt.close();
      }
      catch(SQLException ex2)
      {
        ex2.printStackTrace();
      }
    }
  }

  public void editTreuepunkte(int bestellnr)
  {
    PreparedStatement pstmt = null;
    ResultSet res = null;
    CallableStatement cstmt = null;

    try
    {
      if (dbconn == null)
        dbConnect();

      pstmt = dbconn.prepareStatement(
        "SELECT b.kunde " +
        "FROM Bestellung AS b " +
        "WHERE b.bid = ?");
      pstmt.setInt(1, bestellnr);
      res = pstmt.executeQuery();
      if (!res.next())
        throw new Exception("Kunde konnte nicht gefunden werden");
      int kunde = res.getInt(1);
      res.close();
      pstmt.close();

      cstmt = dbconn.prepareCall("{? = call f_treuepunkte(?)}");
      cstmt.registerOutParameter(1, java.sql.Types.NUMERIC);
      cstmt.setInt(2, bestellnr);
      cstmt.executeUpdate();
      BigDecimal tpunkte = cstmt.getBigDecimal(1);
      cstmt.close();

      if (tpunkte.compareTo(new BigDecimal(-1)) == 0)
        throw new Exception("Treuepunkte konnten nicht berechnet werden");

      pstmt = dbconn.prepareStatement(
        "UPDATE Kunde " +
        "SET tpunkte = ? " +
        "WHERE knr = ?");
      pstmt.setBigDecimal(1, tpunkte);
      pstmt.setInt(2, kunde);
      pstmt.executeUpdate();
      pstmt.close();

      dbconn.commit();
    }
    catch(Exception ex)
    {
      System.out.println(ex.toString());

      try
      {
        if (dbconn != null)
          dbconn.rollback();
        if (cstmt != null)
          cstmt.close();
        if (res != null)
          res.close();
        if (pstmt != null)
          pstmt.close();
      }
      catch(SQLException ex2)
      {
        ex2.printStackTrace();
      }
    }
  }

  public void resetTreuepunkte()
  {
    Statement stmt = null;
    int rowcnt = 0;

    try
    {
      if (dbconn == null)
        dbConnect();

      stmt = dbconn.createStatement();
      rowcnt = stmt.executeUpdate(
        "UPDATE Kunde SET tpunkte = 0 WHERE tpunkte != 0");
      dbconn.commit();
      stmt.close();
    }
    catch(SQLException ex)
    {
      System.out.println(ex.toString());

      try
      {
        if (dbconn != null)
          dbconn.rollback();
        if (stmt != null)
          stmt.close();
      }
      catch(SQLException ex2)
      {
        ex2.printStackTrace();
      }
    }
    finally
    {
      System.out.println("Geaenderte Datensaetze: " + rowcnt);
    }
  }

  public void dbDisconnect()
  {
    try
    {
      if (dbconn != null)
        dbconn.close();
    }
    catch(Exception ex)
    {
      System.out.println(ex.toString());
    }
  }

  public void testBestellungsverwaltung()
  {
    Statement stmt = null;
    ResultSet res = null;

    try
    {
      System.out.println("[ dbOutputKunden() ]");
      System.out.println("knr,vname,nname,tpunkte,ordercnt,sumorder,avgorders");
      dbOutputKunden();
      System.out.println("");

      System.out.println("[ resetTreuepunkte() ]");
      resetTreuepunkte();
      System.out.println("");

      System.out.println("[ dbOutputKunden() ]");
      System.out.println("knr,vname,nname,tpunkte,ordercnt,sumorder,avgorders");
      dbOutputKunden();
      System.out.println("");

      System.out.println("[ call f_treuepunkte_eintragen(bid) ]");
      stmt = dbconn.createStatement();
      stmt.execute("SELECT f_treuepunkte_eintragen(bid) FROM Bestellung ORDER BY bid ASC");
      stmt.close();
      System.out.println("");

      System.out.println("[ dbOutputKunden() ]");
      System.out.println("knr,vname,nname,tpunkte,ordercnt,sumorder,avgorders");
      dbOutputKunden();
      System.out.println("");

      System.out.println("[ resetTreuepunkte() ]");
      resetTreuepunkte();
      System.out.println("");

      stmt = dbconn.createStatement();
      res = stmt.executeQuery("SELECT bid FROM Bestellung ORDER BY bid ASC");
      while (res.next())
      {
        System.out.println("[ editTreuepunkte(" + res.getInt(1) + ") ]");
        editTreuepunkte(res.getInt(1));
      }
      res.close();
      stmt.close();
      System.out.println("");

      System.out.println("[ dbOutputKunden() ]");
      System.out.println("knr,vname,nname,tpunkte,ordercnt,sumorder,avgorders");
      dbOutputKunden();
      System.out.println("");

      dbconn.rollback();
    }
    catch(SQLException ex)
    {
      System.out.println(ex.toString());

      try
      {
        if (dbconn != null)
          dbconn.rollback();
        if (res != null)
          res.close();
        if (stmt != null)
          stmt.close();
      }
      catch(SQLException ex2)
      {
        ex2.printStackTrace();
      }
    }
  }

  public static void main(String[] args)
  {
    Bestellungsverwaltung bv = new Bestellungsverwaltung();
    bv.dbConnect();
    bv.testBestellungsverwaltung();
    bv.dbDisconnect();
  }
}
