-- 5
CREATE OR REPLACE FUNCTION f_before_bestellen()
RETURNS trigger AS '
  DECLARE
    artikel_anz NUMERIC(8) := 0;
  BEGIN
    SELECT SUM(anzahl)
    INTO artikel_anz
    FROM lagern l
    WHERE l.artikel = NEW.partikel
    GROUP BY artikel;

    IF NOT FOUND THEN
      artikel_anz := 0;
    END IF;

    IF TG_OP = ''INSERT'' THEN
      IF artikel_anz = 0 THEN
        RAISE EXCEPTION ''artikel_ausverkauft''
        USING HINT = ''Artikel ist ausverkauft'';
      ELSEIF artikel_anz < NEW.anzahl THEN
        RAISE EXCEPTION ''zu_wenig_stueck''
        USING HINT = ''Zu wenig Artikel vorraetig'';
      END IF;
    END IF;
    RETURN NEW;
  END;
' LANGUAGE plpgsql;

CREATE TRIGGER t_before_bestellen
BEFORE INSERT ON beinhaltet
FOR EACH ROW EXECUTE PROCEDURE f_before_bestellen();

-- 5. test
BEGIN;
INSERT INTO Adresse VALUES
  (DEFAULT,  '8100', 'Koeln 08', 'Kundenstrasse', '8');
INSERT INTO Kunde VALUES
  (DEFAULT, 'vname 08', 'nname 08', currval(pg_get_serial_sequence('Adresse', 'aid')));
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
SAVEPOINT test;

INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-05-02', 'JP', 5200000000000, 1);
ROLLBACK TO SAVEPOINT test;
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-03-01', 'DE', 3100000000000, 1);
ROLLBACK TO SAVEPOINT test;
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-02-02', 'CA', 2200000000000, 41);
ROLLBACK TO SAVEPOINT test;
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 60);
ROLLBACK;

-- 6.1
CREATE OR REPLACE FUNCTION f_bestellwert(bid Bestellung.bid%TYPE)
RETURNS NUMERIC(10,2) AS '
  DECLARE
    gesamtpreis NUMERIC(10,2) := 0;
  BEGIN
    IF bid IS NULL THEN
      RETURN 0;
    END IF;

    SELECT SUM(bein.anzahl * p.preis)
    INTO gesamtpreis
    FROM Bestellung AS b
      JOIN beinhaltet AS bein ON (b.bid = bein.bestellung)
      JOIN Preis AS p ON (bein.pdatum = p.datum AND bein.pland = p.land AND bein.partikel = p.artikel)
    WHERE b.bid = bid
    GROUP BY b.bid;

    IF NOT FOUND THEN
      RAISE EXCEPTION ''bestellung_not_found''
      USING HINT = ''Bestellung konnte nicht gefunden werden'';
    END IF;

    RETURN gesamtpreis;
  EXCEPTION
    WHEN OTHERS THEN
      RAISE NOTICE ''Error: % - %'', SQLSTATE, SQLERRM;
      RETURN -1;
  END;
' LANGUAGE plpgsql;

-- 6.1 test
SELECT kunde, bid, f_bestellwert(bid) AS bestellwert
FROM Bestellung;

-- 6.2
CREATE OR REPLACE FUNCTION f_treuepunkte(bid Bestellung.bid%TYPE)
RETURNS NUMERIC(8,0) AS '
  DECLARE
    treuepunkte NUMERIC(8,0);
    kunde Kunde.knr%TYPE;
    tmp NUMERIC(8,0);
  BEGIN
    IF bid IS NULL THEN
      RETURN 0;
    END IF;

    SELECT b.kunde
    INTO kunde
    FROM Bestellung AS b
    WHERE b.bid = bid;

    IF NOT FOUND THEN
      RAISE EXCEPTION ''kunde_not_found''
      USING HINT = ''Kunde konnte nicht gefunden werden'';
    END IF;

    SELECT f_bestellwert(bid)/10
    INTO treuepunkte;
    --WHERE f_bestellwert(bid) > 0;

    IF NOT FOUND OR treuepunkte IS NULL THEN
      RAISE EXCEPTION ''bestellwert_error''
      USING HINT = ''Bestellwert konnte nicht berechnet werden'';
    END IF;

    IF treuepunkte <= 0 THEN
      RAISE EXCEPTION ''bestellwert_zu_gering''
      USING HINT = ''Bestellwert zu gering'';
    END IF;

    SELECT COUNT(*) AS cnt
    INTO tmp
    FROM Bestellung AS b
    WHERE b.kunde = kunde AND f_bestellwert(b.bid) >= 50
    HAVING COUNT(*) >= 10;

    IF FOUND THEN
      treuepunkte := treuepunkte + 5;
    END IF;

    RETURN treuepunkte;
  EXCEPTION
    WHEN OTHERS THEN
      RAISE NOTICE ''Error: % - %'', SQLSTATE, SQLERRM;
      RETURN -1;
  END;
' LANGUAGE plpgsql;

-- 6.2 test
BEGIN;
INSERT INTO Adresse VALUES
  (DEFAULT,  '8100', 'Koeln 08', 'Kundenstrasse', '8');
INSERT INTO Kunde VALUES
  (DEFAULT, 'vname 08', 'nname 08', currval(pg_get_serial_sequence('Adresse', 'aid')));
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 0);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, DEFAULT, currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 1);

SELECT kunde, bid, f_bestellwert(bid) AS bestellwert, f_treuepunkte(bid) AS treuepunke
FROM Bestellung;
ROLLBACK;

-- 6.3
ALTER TABLE Kunde
ADD COLUMN tpunkte INTEGER NOT NULL DEFAULT 0;

CREATE OR REPLACE FUNCTION f_treuepunkte_eintragen(bestellnr Bestellung.bid%TYPE)
RETURNS void AS '
  DECLARE
    knr Kunde.knr%TYPE;
    treuepunkte NUMERIC(8,0);
  BEGIN
    IF bestellnr IS NULL THEN
      RETURN;
    END IF;

    SELECT b.kunde
    INTO knr
    FROM Bestellung AS b
    WHERE b.bid = bestellnr;

    IF NOT FOUND THEN
      RAISE EXCEPTION ''kunde_not_found''
      USING HINT = ''Kunde konnte nicht gefunden werden'';
    END IF;

    treuepunkte := f_treuepunkte(bestellnr);

    IF treuepunkte = -1 THEN
      RAISE EXCEPTION ''treuepunkte_error''
      USING HINT = ''Treuepunkte konnten nicht berechnet werden'';
    END IF;

    UPDATE Kunde
    SET tpunkte = treuepunkte
    WHERE Kunde.knr = knr;
  EXCEPTION
    WHEN OTHERS THEN
      RAISE NOTICE ''Error: % - %'', SQLSTATE, SQLERRM;
  END;
' LANGUAGE plpgsql;

-- 6.3 test
BEGIN;
SELECT f_treuepunkte_eintragen(bid)
FROM Bestellung
ORDER BY bid ASC;

SELECT k.knr, k.tpunkte, f_treuepunkte(
  (SELECT bid FROM Bestellung AS b WHERE b.kunde = k.knr ORDER BY bid DESC LIMIT 1)
) AS ondemand
FROM Kunde AS k
ORDER BY k.knr;
ROLLBACK;

-- 6.4
CREATE OR REPLACE FUNCTION f_alte_bestellungen_entfernen()
RETURNS void AS '
  DECLARE
    bidnr Bestellung.bid%TYPE;
    rowcnt INTEGER := 0;
    tmp INTEGER;
  BEGIN
    FOR bidnr IN
      SELECT b.bid
      FROM Bestellung as b
      WHERE b.bid NOT IN (
        SELECT bid FROM bezahlt
      ) AND zeitpunkt + interval ''14 days'' <= CURRENT_TIMESTAMP
    LOOP
      DELETE FROM beinhaltet
      WHERE bestellung = bidnr;

      GET DIAGNOSTICS tmp = ROW_COUNT;
      rowcnt = rowcnt + tmp;

      DELETE FROM Bestellung
      WHERE bid = bidnr;

      GET DIAGNOSTICS tmp = ROW_COUNT;
      rowcnt = rowcnt + tmp;
    END LOOP;

    RAISE NOTICE ''Rows deleted: %'', rowcnt;
  EXCEPTION
    WHEN OTHERS THEN
      RAISE NOTICE ''Error: % - %'', SQLSTATE, SQLERRM;
  END;
' LANGUAGE plpgsql;

-- 6.4 test
BEGIN;
INSERT INTO Adresse VALUES
  (DEFAULT,  '8100', 'Koeln 08', 'Kundenstrasse', '8');
INSERT INTO Kunde VALUES
  (DEFAULT, 'vname 08', 'nname 08', currval(pg_get_serial_sequence('Adresse', 'aid')));
INSERT INTO Bestellung VALUES
  (DEFAULT, CURRENT_TIMESTAMP - interval '14 days', currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 0),
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-02-01', 'CA', 2100000000000, 1);
INSERT INTO Bestellung VALUES
  (DEFAULT, CURRENT_TIMESTAMP - interval '13 days', currval(pg_get_serial_sequence('Kunde', 'knr')));
INSERT INTO beinhaltet VALUES
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-01-01', 'US', 1100000000000, 0),
  (currval(pg_get_serial_sequence('Bestellung', 'bid')), '1980-02-01', 'CA', 2100000000000, 1);

SELECT f_alte_bestellungen_entfernen();
ROLLBACK;
