-- 1.1 Standorte
SELECT
  s.sid,s.bezeichnung, a.svnr, a.vname, a.nname,
  CASE WHEN s.leiter = a.svnr THEN 'Leiter'  ELSE 'Angestellter' END AS position
FROM Standort AS s
  JOIN Angestellter AS a ON (s.sid = a.standort)
ORDER BY s.sid, a.nname, a.vname;

-- 1.2 bester Kunde
SELECT k.knr, k.vname, k.nname, SUM(bein.anzahl * p.preis) AS gesamtbestellsumme
FROM Bestellung AS b
  JOIN bezahlt AS bez ON (b.bid = bez.bid)
  JOIN beinhaltet AS bein ON (b.bid = bein.bestellung)
  JOIN Preis AS p ON (bein.pdatum = p.datum AND bein.pland = p.land AND bein.partikel = p.artikel)
  JOIN Kunde AS k ON (k.knr = b.kunde)
GROUP BY k.knr, k.vname, k.nname
HAVING SUM(bein.anzahl * p.preis) >= ALL (
  SELECT SUM(bein.anzahl * p.preis)
  FROM Bestellung AS b
    JOIN bezahlt AS bez ON (b.bid = bez.bid)
    JOIN beinhaltet AS bein ON (b.bid = bein.bestellung)
    JOIN Preis AS p ON (bein.pdatum = p.datum AND bein.pland = p.land AND bein.partikel = p.artikel)
  GROUP BY b.kunde
);

-- 1.3 Standort-Ranking
SELECT s.sid, s.bezeichnung, count(a.svnr) AS mitarbeiter
FROM Standort AS s
  JOIN Angestellter AS a ON (s.sid = a.standort)
GROUP BY s.sid, s.bezeichnung
ORDER BY mitarbeiter DESC, s.bezeichnung;

-- 2
WITH RECURSIVE serietmp(vorgaenger, nachfolger) AS (
    SELECT vorgaenger, nachfolger FROM Serie WHERE vorgaenger = 3100000000000
  UNION ALL
    SELECT s1.vorgaenger, s1.nachfolger
    FROM Serie AS s1, serietmp AS s2
    WHERE s1.vorgaenger = s2.nachfolger
)
SELECT * FROM serietmp;

-- 3.1
SELECT ean, bezeichnung
FROM Artikel
WHERE ean NOT IN (
  SELECT partikel
  FROM Bestellung AS b
    JOIN beinhaltet AS bein ON (b.bid = bein.bestellung)
  WHERE b.kunde = 1000021
);

-- 3.2
SELECT DISTINCT k.knr, k.vname, k.nname
FROM Kunde AS k
WHERE NOT EXISTS (
  SELECT ean, bezeichnung
  FROM Artikel
  WHERE ean NOT IN (
    SELECT partikel
    FROM Bestellung AS b
      JOIN beinhaltet AS bein ON (b.bid = bein.bestellung)
    WHERE b.kunde = k.knr
  )
);

-- 4
CREATE VIEW tagespreise_view AS
SELECT a.ean, a.bezeichnung, a.typ, p.datum, p.preis, p.steuer, l.code, l.name, l.kontinent
FROM Artikel AS a
  JOIN Preis AS p ON (a.ean = p.artikel)
  JOIN Land AS l ON (p.land = l.code);

SELECT * FROM tagespreise_view;

DROP VIEW tagespreise_view;
